import os
import subprocess

try:
    subprocess.call(["md5sum", "--help"], stdout=subprocess.DEVNULL)
except FileNotFoundError:
	raise Exception # md5sum not found. Are you using Linux?

def md5sum(filename):
	"""
		returns the md5sum of a file, along with its filename, e.g.:
		0df61fe4ddf4455ba4d4e3c15abfabe2  predict_draft.py
	"""
	return os.popen(f'md5sum \'{filename}\'').read().split(' ')[0]

def md5sum_folder(folder_name):
	"""
		returns the md5sum of a file using the command `tar -cf - {folder_name} | md5sum`
	"""
	
	return os.popen(f'tar -cf - \'{folder_name}\' | md5sum').read()

if __name__ == '__main__':
	print(md5sum('predict_draft.py'))


import cv2
import numpy as np
import tifffile
import os
import glob
from scipy.ndimage import geometric_transform
import tifffile
import glob
import numpy as np
import matplotlib.pyplot as plt


def generate_coupled_image(movie, frame, output):
    """
    Generates an image from the previous frame of the given frame and the given frame of the given movie. Should be used as input to Siam_UNet. The output image is a tiff file with the files like this, with the output image of twice the width of the input.
    [    Previous   |     Frame to be    ]
    [     Frame     |   Inferred From    ]

    Note on arg: 
        frame (int): 0-indexed
    """

    curr_frame = tifffile.imread(movie, key=frame)
    if frame == 0:
        prev_frame = tifffile.imread(movie, key=(frame + 1))
    else:
        prev_frame = tifffile.imread(movie, key=(frame - 1))

    if curr_frame is None:
        print(f'Error: {movie} not found.')
        raise IOError  # tiff file not found

    out = np.concatenate((prev_frame, curr_frame), axis=1).astype(np.uint8)
    cv2.imwrite(filename=output, img=out, )


def generate_coupled_image_from_self(img, out_img, noise_amp=10):
    """
        Generates an input image for siam by concatenating an image with a transformed version of itself 
    """

    def __synthesize_prev_img(in_img, noise_amp=10):
        """Synthesizes previous frame by transforming the input image

        Args:
            in_img (str): input image path
            noise_amp (int, optional): Defaults to 10.

        Returns:
            2-D ndarray: the synthesized previous image
        """
        data = tifffile.imread(in_img)
        image = data
        modes_x, modes_y = 10, 4
        amp = 1
        amps_x, amps_y = np.random.random_sample(modes_x) * amp, np.random.random_sample(modes_y) * amp

        def func(xy):
            return (xy[0] + np.sum(amps_y * np.sin(modes_y * 2 * np.pi * xy[0] / image.shape[0])),
                    xy[1] + np.sum(amps_x * np.sin(modes_x * 2 * np.pi * xy[1] / image.shape[1])))

        out = geometric_transform(image, func)
        noise = np.random.normal(0, noise_amp, size=image.shape)
        out = out + noise
        out[out < 0] = 0
        out[out > 255] = 255

        return out

    curr_frame = tifffile.imread(img)
    synthesized_previous_frame = __synthesize_prev_img(img, noise_amp)

    if curr_frame is None:
        raise IOError  # tiff file not found

    out = np.concatenate((synthesized_previous_frame, curr_frame), axis=1).astype(np.uint8)
    cv2.imwrite(filename=out_img, img=out, )


def utilize_search_result(search_result_mr_txt, movie_path_prefix, labels_path_prefix, output_folder):
    """Parses search results obtained from find_frame_of_image.find_frame_of_image()'s machine readable output and pass them to generate_coupled_image() to create training data for Siam_UNet. 

    This function outputs two folders under output_folder: image and label. All the output will have a suffix of .tif, and thyey will have the same names as the names of input labels. The output in the image directory will be the current frame concatenated with the previous frame, while the label directory will contain the corresponding labels, directly copied from the input labels directory. Visually, the directory structure at output:

    output_folder
    |
    |----image
    |       |-----concatenated images
    |
    |
    |----label
    |       |-----labels copied from the input labels directory

    Args:
        search_result_mr_txt (str): path to the machine readable output
        movie_path_prefix, labels_path_prefix (str): prefixes to name of each movie in the search result file, respectively
        output_folder (str): path to the output folder. See above.
    """

    image_output_folder = output_folder + '/image/'
    label_output_folder = output_folder + '/label/'

    # make output folder if it does not exist
    os.system(f'mkdir -p \'{output_folder}\'')
    # recreate image and label output folders
    os.system(f'rm -rf \'{image_output_folder}\'')
    os.system(f'rm -rf \'{label_output_folder}\'')
    os.system(f'mkdir -p \'{image_output_folder}\'')
    os.system(f'mkdir -p \'{label_output_folder}\'')

    with open(search_result_mr_txt) as sr:
        lines = sr.readlines()
        for line in lines:
            label_path = labels_path_prefix + '/' + line.split('\t')[0]
            image_path = movie_path_prefix + '/' + line.split('\t')[1]
            output_image_path = image_output_folder + line.split('\t')[0]
            frame_number = int(line.split('\t')[2])
            # copy label to label folder
            os.system(f'cp \'{label_path}\' \'{label_output_folder}\'')
            # create concatenated image in image_folder
            generate_coupled_image(image_path, frame=frame_number, output=output_image_path)

    for file in glob.glob(f'{label_output_folder}/*.tif'):
        i = cv2.imread(file)
        fo = cv2.cvtColor(i, cv2.COLOR_RGB2GRAY)
        os.remove(file)
        cv2.imwrite(file, fo)


import tifffile
from tifffile import TiffFile

def fetch_frame(tif_file):
    """Returns a generator of frames in a tif file

    Args:
        tif_file (str): path to the tif file of concern

    Yields:
        numpy ndarray: A matrix representing a frame in the image. Should be two dimensional if the input is grayscale. You can also quickly get its dimension with tif_key.pages[0].shape

    """
    tif_key = TiffFile(tif_file)
    tif_len = len(tif_key.pages)
    for i in range(0, tif_len):
        yield tifffile.imread(tif_file, key=i)

def individual_tif_generator(dir, tif_len):
    """
    A generator that returns each frame in a directory. See siam_unet_cosh.py for usage.
    """
    for i in range(tif_len):
        yield tifffile.imread(f'{dir}/{i}.tif')
    
    """
    And now, because tifffile supports a iterable object as its data parameter, you can do

        tifffile.imwrite(data=tqdm(self.individual_tif_generator(dir=temp_dir), total=self.tif_len, unit='frame'), file=self.result_name, dtype=np.uint8, shape=self.imgs_shape)
    
    Again, see siam_unet_cosh.py for usage.
    """


if __name__ == '__main__':
    ## A demonstration of fetch_frame()
    import cv2
    import constants
    ###
    # Now instead of
    #  for frame in movie:
    # You can do
    for frame in fetch_frame(f'{constants.RAZER_LOCAL_MOVIE_DIR}/{constants.TIFS_NAMES[0]}.tif'):
        cv2.imshow('i', frame)
        cv2.waitKey(0)



try:
    import torch
except ImportError:
    raise ImportError # Torch is not installed. Please install through https://pytorch.org/get-started/locally/

from .data import DataProcess
from .train import Trainer
from .predict import Predict
from .unet import Unet
from .attention_unet import AttentionUnet
from .losses import *




from .cuda_test import get_gpu_name # type: ignore
from .__cpu_count__ import get_cpu_count
from .__md5sum__ import md5sum
import os
import time
import platform
import subprocess
if platform.system() != 'Linux':
    raise Exception  # this script is designed to use Linux bash commands. Please use Linux

def get_hostname():
	return os.popen(f'hostname').read()

def get_memory_per_node():
	return os.getenv('SLURM_MEM_PER_NODE')

def get_cpu_model_name():
	raw_info = os.popen(f'lscpu | grep -i \'Model name\'').read()
	raw_info = raw_info[raw_info.find(':') + 1:].strip()
	return raw_info

def get_info_file_header():
	header = ''
	header += f'Time: {time.ctime()}\n'
	header += f'Node name: {get_hostname()}'
	header += f'CPU Model: {get_cpu_model_name()}\n'
	header += f'Core count: {get_cpu_count()}\n'
	header += f'Memory allocation: {get_memory_per_node()}\n'
	header += f'GPU: {get_gpu_name()}\n'
	header += '------------------------\n'
	return header

def write_info_file(outfile_name, additional_info):
	with open(outfile_name, 'w') as f:
		f.write(f'Info file name: {outfile_name}\n')
		f.write(get_info_file_header())
		f.write(additional_info)

def zoom_in(input, width, height, x=None, y=None, output='crop.mp4'):
	if x is None or y is None: # center video
		os.system(f'ffmpeg -i \'{input}\' -y -filter:v "crop={width}:{height}" \'{output}\'')
	else:	
		os.system(f'ffmpeg -i \'{input}\' -y -filter:v "crop={width}:{height}:{x}:{y}" \'{output}\' 2>/dev/null')

def create_zoomed_in_comparison(file1, file2, width, height, x=None, y=None, output='compare.mp4'):
	temp_1 = 'temp_' + file1.split('/')[-1]
	temp_2 = 'temp_' + file2.split('/')[-1]
	zoom_in(file1, width, height, x, y, output=temp_1)
	zoom_in(file2, width, height, x, y, output=temp_2)
	vertically_stack_two_videos(temp_1, temp_2, output)
	os.system(f'rm \'{temp_1}\'')
	os.system(f'rm \'{temp_2}\'')

def vertically_stack_two_videos(video1, video2, output):
	try:
		subprocess.run(["ffmpeg"], stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
	except:
		raise Exception   # ffmpeg not detected. Please install ffmpeg
	ffmpeg_command = f'ffmpeg -y -i \'{video1}\' -i \'{video2}\' -filter_complex vstack=inputs=2 \'{output}\''
	os.system(ffmpeg_command)
	with open(output + '.info.txt', 'w') as f:
		f.write(f'{get_info_file_header()}\nOperation: two stack\nOutfile name: {output}\nVideos:\n{video1}\t{video2}\n')


def grid_four_videos(video1, video2, video3, video4, output):
	try:
		subprocess.run(["ffmpeg"], stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
	except:
		raise Exception   # ffmpeg not detected. Please install ffmpeg
	ffmpeg_command = f'ffmpeg -y \
		-i \'{video1}\' -i \'{video2}\' -i \'{video3}\' -i \'{video4}\' \
		-filter_complex \
		"[0:v][1:v]hstack=inputs=2[top]; \
		[2:v][3:v]hstack=inputs=2[bottom]; \
		[top][bottom]vstack=inputs=2[v]" \
		-map "[v]" \
		\'{output}\''
	os.system(f'{ffmpeg_command}')
	with open(output + '.info.txt', 'w') as f:
		f.write(f'{get_info_file_header()}\nOperation: four stack\nOutfile name: {output}\nVideos:\n{video1}\t{video2}\n{video3}\t{video4}\n')

def grid_six_videos(video1, video2, video3, video4, video5, video6, output):
	try:
		subprocess.run(["ffmpeg"], stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
	except:
		raise Exception   # ffmpeg not detected. Please install ffmpeg
	ffmpeg_command = f'ffmpeg -y -i \'{video1}\' -i \'{video2}\' \
		-i \'{video3}\' -i \'{video4}\' \
		-i \'{video5}\' -i \'{video6}\' \
		-filter_complex \
		"[0:v][1:v][2:v]hstack=inputs=3[top];\
		[3:v][4:v][5:v]hstack=inputs=3[bottom];\
		[top][bottom]vstack=inputs=2[v]" \
		-map "[v]" \
		\'{output}\''
	os.system(f'{ffmpeg_command}')
	with open(output + '.info.txt', 'w') as f:
		f.write(f'{get_info_file_header()}\nOperation: six stack\nOutfile name: {output}\nVideos:\n{video1}\t{video2}\n{video3}\t{video4}\n{video5}\t{video6}\n')




if __name__ == '__main__':
	pass


import cv2
import os
os.nice(20)
import subprocess
import numpy as np
import tifffile
from skimage import morphology

import platform
if platform.system() != 'Linux':
    raise Exception  # this script is designed to use Linux bash commands. Please use Linux

try:
    subprocess.run(["ffmpeg"], stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
except:
    raise Exception   # ffmpeg not detected. Please install ffmpeg


def convert_to_mp4(tiff_movie, output_file="out.mp4", fps=30, perform_threshold=False, threshold_val=250, invert=False, normalize_to_0_255=True, closing=False, close_thresh=10):
    """
    Converts tiff_movie to an mp4 movie

    params:
        tiff_movie: file name for input
        fps: number of frames per second in the output
        perform_threshold: whether we want the images to be thresholded (i.e. the pixels to be aligned to either white or black), 0 is black, 255 is white
        threshold_val: the value at which thresholding is done. pixel_val = 255 if pixel_val >= threshold_val else 0
        normalize_to_0_255: scales each individual image to the full range of (0, 255).
    """

    imgs = frame_generator(tiff_movie)
    temp_dir = f'temp_{tiff_movie.split("/")[-1]}'
    os.system(f"mkdir -p \'{temp_dir}\'")

    ct = 0
    for img in imgs:
        if invert:
            img = np.ones(img.shape) * 255 - img
        if perform_threshold:
            img = ((img >= threshold_val) * np.ones(img.shape) * 255 ).astype(np.uint8)
        if normalize_to_0_255:
            img = img.astype(np.double)
            img = (img - np.min(img))/(np.max(img) - np.min(img)) * 255
            img = img.astype(np.uint8)
        if closing:
            img = morphology.opening(img, selem=morphology.star(close_thresh))
        name = str(ct).zfill(5)
        cv2.imwrite(f"{temp_dir}/{name}.png", img, [cv2.IMWRITE_PNG_COMPRESSION, 0])
        ct += 1

    # convert to mp4
    # -crf 17 is visually "lossless" as mentioned in https://trac.ffmpeg.org/wiki/Encode/H.264
    ffmpeg_command = f'ffmpeg -y -r {fps} -i \'{temp_dir}/%5d.png\' -c:v libx264 -crf 17 -pix_fmt yuv420p \'{output_file}\''

    # execution and cleanup
    print('Executing ' + ffmpeg_command)
    os.system(f'{ffmpeg_command} ; rm -rf \'{temp_dir}\'')

def frame_generator(tiff_movie):
    """Returns a generator for each individual frame of the movie. Should function in the same way as imgs.

    Args:
        tiff_movie (str): path to tiff movie
    """
    tif_key = tifffile.TiffFile(tiff_movie)
    tif_len = len(tif_key.pages)
    for i in range(tif_len):
        yield tifffile.imread(tiff_movie, key=i)


if __name__ == '__main__':
    # convert_to_mp4('/media/longyuxi/H is for HUGE/docmount backup/unet_pytorch/training_data/test_data/new_microscope/21B11-shgGFP-kin-18-bro4.tif', output_file='/media/longyuxi/H is for HUGE/docmount backup/predicted_out/siam_bce_predicted_lowmem_21B11-shgGFP-kin-18-bro4.mp4', normalize_to_0_255=True)
    convert_to_mp4('/media/longyuxi/H is for HUGE/docmount backup/predicted_out/minitest.tif', output_file='/media/longyuxi/H is for HUGE/docmount backup/predicted_out/minitest_closing.mp4', normalize_to_0_255=True, closing=True)
    


import cv2 # type: ignore
import numpy as np

def extract_frame_of_movie(tiff_movie, frame_number, output_file):
	"""Extract a certain frame of tiff_movie, and write it into a separate file. Used for testing find_frame_of_image

	Args:
		tiff_movie (str): path to input
		frame_number (int): which frame to extract

	Raises:
		IOError: raised when tiff file cannot be found
	"""
	retval, imgs = cv2.imreadmulti(tiff_movie)
	if not retval:
		print(f'Error: {tiff_movie} not found.')
		raise IOError   # tiff file not found
	output = imgs[frame_number]
	print(output.shape)
	cv2.imwrite(filename=output_file, img=output, )

def extract_frames_of_movie(tiff_movie, frame_number, output_file):
	"""Extract up to a certain frame of tiff_movie, and write it into a separate file. Used for testing find_frame_of_image

	Args:
		tiff_movie (str): path to input
		frame_number (int): up to which frame to extract

	Raises:
		IOError: raised when tiff file cannot be found
	"""
	retval, imgs = cv2.imreadmulti(tiff_movie)
	if not retval:
		print(f'Error: {tiff_movie} not found.')
		raise IOError   # tiff file not found
	output = imgs[:frame_number]
	cv2.imwritemulti(filename=output_file, img=output, )


if __name__ == '__main__':
	# extract_frames_of_movie(f'{constants.LOCAL_MOVIE_DIR}/{constants.TIFS_NAMES[1]}.tif', 10, f'{constants.LOCAL_OUT_DIR}/{constants.TIFS_NAMES[1]}_up_to_frame_10.tif')
	pass


from typing import Union

import numpy as np
import tifffile
import torch

from .unet3d import UNet3D
from ..progress import ProgressNotifier
from ..utils import save_as_tif, get_device


class Predict:
    """
    Class for prediction of movies or 3D stacks with 3D U-Net

    1) Loading file and preprocess (normalization)
    2) Resizing of images into patches with resize_dim
    3) Prediction with U-Net
    4) Stitching of predicted patches and averaging of overlapping regions

    Parameters
    ----------
    vol : ndarray, string
        volume to predict, if string, attempt to load from tif file
    result_name : str
        path for result
    model_params : str
        path of u-net parameters (.pth file)
    network
        Network class
    resize_dim
        Image dimensions for resizing for prediction
    invert : bool
        Invert greyscale of image(s) before prediction
    normalization_mode : str
        Mode for intensity normalization for 3D stacks prior to prediction ('single': each image individually,
        'all': based on histogram of full stack, 'first': based on histogram of first image in stack)
    clip_threshold : Tuple[float, float]
        Clip threshold for image intensity before prediction
    add_patch : int, optional
        Add additional patches for splitting large images to increase overlap
    normalize_result : bool
        If true, results are normalized to [0, 255]
    progress_bar : bool, optional
        Whether to display progress bar during prediction.
    device : torch.device, optional
        Device to run pytorch model on. Default is 'auto', which selects CUDA or MPS if available.
    progress_notifier:
        Wrapper to show tqdm progress notifier in gui
    """

    def __init__(self, vol, result_name, model_params, network=UNet3D, resize_dim=(512, 512),
                 invert=False, normalization_mode='single', clip_threshold=(0., 99.8), add_patch=0,
                 normalize_result=False, progress_bar=True, device: Union[torch.device, str] = 'auto',
                 progress_notifier: ProgressNotifier = ProgressNotifier.progress_notifier_tqdm()):

        if isinstance(vol, str):
            vol = tifffile.imread(vol)

        if device == 'auto':
            self.device = get_device()
        else:
            self.device = torch.device(device)

        self.resize_dim = resize_dim
        self.add_patch = add_patch
        self.normalize_result = normalize_result
        self.invert = invert
        self.normalization_mode = normalization_mode
        self.clip_threshold = clip_threshold
        self.result_name = result_name
        self.progress_bar = progress_bar

        # read, preprocess and split data
        vol = self.__reshape_data(vol)
        vol = self.__preprocess(vol)
        patches = self.__split(vol)
        del vol

        # load model and predict data
        self.model_params = torch.load(model_params, map_location=self.device)
        if 'use_interpolation' not in self.model_params.keys():
            self.model_params['use_interpolation'] = False
        self.model = network(n_filter=self.model_params['n_filter'], in_channels=self.model_params['in_channels'],
                             out_channels=self.model_params['out_channels'],
                             use_interpolation=self.model_params['use_interpolation']).to(self.device)
        self.model.load_state_dict(self.model_params['state_dict'])
        self.model.eval()
        result_patches = self.__predict(patches, progress_notifier)
        del patches, self.model

        # stitch patches (mean of overlapped regions)
        vol_result = self.__stitch(result_patches)
        del result_patches, self.model_params

        # save as .tif file
        save_as_tif(vol_result, self.result_name, normalize=normalize_result)
        del vol_result
        if torch.cuda.is_available():
            torch.cuda.empty_cache()

    def __reshape_data(self, vol):
        self.vol_shape = vol.shape
        if len(self.vol_shape) == 2:  # if single image
            vol = np.expand_dims(vol, axis=0)
            self.vol_shape = vol.shape
        return vol

    def __preprocess(self, vol):
        clip_threshold = (np.nanpercentile(vol, self.clip_threshold[0]),
                          np.percentile(vol, self.clip_threshold[1]))
        vol = np.clip(vol, clip_threshold[0], clip_threshold[1])
        vol = vol - np.min(vol)
        vol = vol / np.max(vol) * 255
        if self.invert:
            vol = 255 - vol
        return vol

    def __split(self, vol):
        # number of patches in x and y
        self.N_z = int(np.ceil(self.vol_shape[0] / self.resize_dim[0])) + self.add_patch
        self.N_x = int(np.ceil(self.vol_shape[1] / self.resize_dim[1])) + self.add_patch
        self.N_y = int(np.ceil(self.vol_shape[2] / self.resize_dim[2])) + self.add_patch
        self.N_x += self.add_patch if self.N_z > 1 else 0
        self.N_x += self.add_patch if self.N_x > 1 else 0
        self.N_y += self.add_patch if self.N_y > 1 else 0

        self.N = self.N_x * self.N_y * self.N_z  # total number of patches

        # define array for prediction
        patches = np.zeros((self.N, self.resize_dim[0], self.resize_dim[1], self.resize_dim[2]), dtype='uint8')

        # zero padding of image if vol_shape < resize_dim
        z_gap = max(0, self.resize_dim[0] - self.vol_shape[0])
        x_gap = max(0, self.resize_dim[1] - self.vol_shape[1])
        y_gap = max(0, self.resize_dim[2] - self.vol_shape[2])
        vol = np.pad(vol, ((0, z_gap), (0, x_gap), (0, y_gap)), 'reflect')

        # starting indices of patches
        self.Z_start = np.linspace(0, self.vol_shape[0] - self.resize_dim[0], self.N_z).astype('uint16')
        self.X_start = np.linspace(0, self.vol_shape[1] - self.resize_dim[1], self.N_x).astype('uint16')
        self.Y_start = np.linspace(0, self.vol_shape[2] - self.resize_dim[2], self.N_y).astype('uint16')

        # split in resize_dim
        n = 0
        for j in range(self.N_z):
            for k in range(self.N_x):
                for p in range(self.N_y):
                    patches[n, :, :, :] = vol[self.Z_start[j]:self.Z_start[j] + self.resize_dim[0],
                                          self.X_start[k]:self.X_start[k] + self.resize_dim[1],
                                          self.Y_start[p]:self.Y_start[p] + self.resize_dim[2]]
                    n += 1
        return patches

    def __predict(self, patches, progress_notifier: ProgressNotifier = ProgressNotifier.progress_notifier_tqdm()):
        result_patches = np.zeros_like(patches, dtype='uint8')
        print('Predicting data ...') if self.progress_bar else None
        with torch.no_grad():
            _progress_notifier = enumerate(progress_notifier.iterator(patches)) if self.progress_bar else enumerate(
                patches)
            for i, patch_i in _progress_notifier:
                patch_i = torch.from_numpy(patch_i.astype('float32') / 255).to(self.device).view((1, 1,
                                                                                                  self.resize_dim[0],
                                                                                                  self.resize_dim[1],
                                                                                                  self.resize_dim[2]))
                res_i, logits_i = self.model(patch_i)
                res_i = res_i.view(
                    (self.resize_dim[0], self.resize_dim[1], self.resize_dim[2])).cpu().numpy()
                result_patches[i] = (res_i * 255).astype('uint8')
                del patch_i, res_i
        return result_patches

    def __stitch(self, result_patches):
        # create array
        _vol_result = np.zeros(shape=(3, max(self.vol_shape[0], self.resize_dim[0]),
                                      max(self.resize_dim[1], self.vol_shape[1]),
                                      max(self.resize_dim[2], self.vol_shape[2])), dtype='float16') * np.nan

        n = 0
        for i in range(self.N_z):
            for j in range(self.N_x):
                for k in range(self.N_y):
                    _vol_result[np.mod(n, 3),
                    self.Z_start[i]:self.Z_start[i] + self.resize_dim[0],
                    self.X_start[j]:self.X_start[j] + self.resize_dim[1],
                    self.Y_start[k]:self.Y_start[k] + self.resize_dim[2]] = result_patches[n]
                    n += 1

        # average overlapping regions
        vol_result = np.nanmean(_vol_result, axis=0).astype('uint8')

        # change to input size (if zero padding)
        vol_result = vol_result[:self.vol_shape[0], :self.vol_shape[1], :self.vol_shape[2]]

        return np.squeeze(vol_result)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['bio_image_unet',
 'bio_image_unet.progress',
 'bio_image_unet.siam_unet',
 'bio_image_unet.siam_unet.helpers',
 'bio_image_unet.unet',
 'bio_image_unet.unet3d',
 'bio_image_unet.utils']

package_data = \
{'': ['*']}

install_requires = \
['albumentations>=1.3,<2.0',
 'napari>=0.4.18,<0.5.0',
 'numpy>=1.20,<2.0',
 'packaging',
 'scikit-image>0.18,<1.0',
 'tifffile',
 'torch>=2.0.0,<3.0.0',
 'tqdm>=4.61,<5.0']

setup_kwargs = {
    'name': 'bio-image-unet',
    'version': '1.0.0b2',
    'description': 'Implementations of U-Net, Siam U-Net and 3D U-Net for biological image segmentation',
    'long_description': '# Bio Image U-Net\n\nImplementations of U-Net, Siam U-Net and 3D U-Net for biological image segmentation\n\n\n## Installation\n### PyPI\n``pip install bio-image-unet``\n### GitHub\n``pip install git+https://github.com/danihae/bio-image-unet``\n\n## Usage example\nImport package with ``import bio_image_unet``\n\n[iPython Notebook for getting started with U-Net](https://github.com/danihae/bio-image-unet/blob/master/using_unet.ipynb) \\\n[iPython Notebook for getting started with Siam U-Net](https://github.com/danihae/bio-image-unet/blob/master/using_siam_unet.ipynb)\n',
    'author': 'Daniel Haertter',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/danihae/bio-image-unet',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



try:
    import torch
except ImportError:
    raise ImportError('Torch is not installed. Please install through https://pytorch.org/get-started/locally/')

# check if torch version >= 1.9.0
from packaging import version

if not version.parse(torch.__version__) >= version.parse('1.9.0'):
    raise ImportError(
        f'This package requires torch >=1.9.0 to run. You have version {torch.__version__}.Please install through https://pytorch.org/get-started/locally/')

from .data import DataProcess
from .losses import *
from .predict import Predict
from .siam_unet import Siam_UNet
from .train import Trainer
